/*
 * This file ("RenderBatteryBox.java") is part of the Actually Additions mod for Minecraft.
 * It is created and owned by Ellpeck and distributed
 * under the Actually Additions License to be found at
 * http://ellpeck.de/actaddlicense
 * View the source code at https://github.com/Ellpeck/ActuallyAdditions
 *
 * © 2015-2017 Ellpeck
 */

package de.ellpeck.actuallyadditions.mod.blocks.render;

import com.mojang.blaze3d.matrix.MatrixStack;
import de.ellpeck.actuallyadditions.mod.ActuallyAdditions;
import de.ellpeck.actuallyadditions.mod.items.ItemBattery;
import de.ellpeck.actuallyadditions.mod.tile.TileEntityBatteryBox;
import de.ellpeck.actuallyadditions.mod.util.Lang;
import net.minecraft.client.Minecraft;
import net.minecraft.client.gui.FontRenderer;
import net.minecraft.client.renderer.IRenderTypeBuffer;
import net.minecraft.client.renderer.model.ItemCameraTransforms;
import net.minecraft.client.renderer.tileentity.TileEntityRenderer;
import net.minecraft.client.renderer.tileentity.TileEntityRendererDispatcher;
import net.minecraft.item.BlockItem;
import net.minecraft.item.ItemStack;
import net.minecraft.util.Util;
import net.minecraft.util.math.vector.Vector3f;
import net.minecraftforge.api.distmarker.Dist;
import net.minecraftforge.api.distmarker.OnlyIn;
import net.minecraftforge.energy.CapabilityEnergy;

@OnlyIn(Dist.CLIENT)
public class RenderBatteryBox extends TileEntityRenderer<TileEntityBatteryBox> {
    public RenderBatteryBox(TileEntityRendererDispatcher rendererDispatcherIn) {
        super(rendererDispatcherIn);
    }

    // TODO: [port] migrate to matric (see cleanstart) (done partly)
    @Override
    public void render(TileEntityBatteryBox tile, float partialTicks, MatrixStack matrices, IRenderTypeBuffer buffer, int combinedLight, int combinedOverlay) {
        ItemStack stack = tile.inv.getStackInSlot(0);
        if (stack.isEmpty() || !(stack.getItem() instanceof ItemBattery)) {
            return;
        }

        matrices.push();
        matrices.translate(.5f, .35f, .5f);
        matrices.rotate(Vector3f.ZP.rotationDegrees(180));

        matrices.push();
        matrices.scale(0.0075F, 0.0075F, 0.0075F);
        matrices.translate(0F, 0F, -60F);

        stack.getCapability(CapabilityEnergy.ENERGY).ifPresent(cap -> {
            FontRenderer font = Minecraft.getInstance().fontRenderer;

            String energyTotal = Lang.cleanEnergyValues(cap, false);
            String energyName = Lang.transI18n("misc", "power_name_long");

            for (int i = 0; i < 4; i++) {
                font.drawString(matrices, energyTotal, -font.getStringWidth(energyTotal) / 2F, 10F, 0xFFFFFF);
                font.drawString(matrices, energyName, -font.getStringWidth(energyName) / 2F, 20F, 0xFFFFFF);

                matrices.translate(-60F, 0F, 60F);
                matrices.rotate(Vector3f.YP.rotationDegrees(90));
            }
            //            TODO: Remove if the above works
            //            NumberFormat format = NumberFormat.getInstance();
            //            FontRenderer font = Minecraft.getInstance().fontRenderer;
            //
            //            String s = format.format(cap.getEnergyStored()) + "/" + format.format(cap.getMaxEnergyStored());
            //            float lengthS = -font.getStringWidth(s) / 2F;
            //            String s2 = I18n.format("actuallyadditions.cflong");
            //            float lengthS2 = -font.getStringWidth(s2) / 2F;
            //
            //            for (int i = 0; i < 4; i++) {
            //                font.drawString(matrices, s, lengthS, 10F, 0xFFFFFF);
            //                font.drawString(matrices, s2, lengthS2, 20F, 0xFFFFFF);
            //
            //                RenderSystem.translatef(-50F, 0F, 50F);
            //                RenderSystem.rotatef(90F, 0F, 1F, 0F);
            //            }
        });

        matrices.pop(); // text rotation
        matrices.pop(); // rotation + centering

        double boop = Util.milliTime() / 800D;
        float scale = stack.getItem() instanceof BlockItem
            ? 0.85F
            : 0.65F;

        matrices.push();
        matrices.translate(.5f, 1f + Math.sin(boop % (2 * Math.PI)) * 0.065, .5f);
        matrices.rotate(Vector3f.YP.rotationDegrees((float) (boop * 40D % 360)));
        matrices.scale(scale, scale, scale);

        try {
            Minecraft.getInstance().getItemRenderer().renderItem(
                stack, ItemCameraTransforms.TransformType.FIXED, combinedLight, combinedOverlay, matrices, buffer
            );
        } catch (Exception e) {
            ActuallyAdditions.LOGGER.error("Something went wrong trying to render an item in a battery box! The item is " + stack.getItem().getRegistryName() + "!", e);
        }

        matrices.pop();
    }
}
